# Generated by Django 3.0.9 on 2020-12-06 20:52

import uuid

import _socket
import django.core.validators
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_crypto_fields.fields.encrypted_char_field
import django_crypto_fields.fields.encrypted_text_field
import django_crypto_fields.fields.firstname_field
import django_crypto_fields.fields.lastname_field
import django_revision.revision_field
import edc_model.models.fields.other_charfield
import edc_model_fields.fields.other_charfield
import edc_screening.model_mixins.screening_fields_model_mixin
import edc_sites.models
import edc_utils.date
import simple_history.models
from django.conf import settings
from django.db import migrations, models

import mocca_screening.mocca_original_sites
import mocca_screening.models.mocca_register
import mocca_screening.models.mocca_register_contact


class Migration(migrations.Migration):

    replaces = [
        ("mocca_screening", "0001_initial"),
        ("mocca_screening", "0002_auto_20201203_0338"),
        ("mocca_screening", "0003_auto_20201203_1610"),
        ("mocca_screening", "0004_auto_20201203_1757"),
        ("mocca_screening", "0005_auto_20201204_0217"),
        ("mocca_screening", "0006_moccaregistercontact"),
        ("mocca_screening", "0007_auto_20201204_0232"),
        ("mocca_screening", "0008_auto_20201204_0312"),
        ("mocca_screening", "0009_auto_20201204_0323"),
        ("mocca_screening", "0010_auto_20201204_0330"),
        ("mocca_screening", "0011_auto_20201204_0347"),
        ("mocca_screening", "0012_auto_20201204_0352"),
        ("mocca_screening", "0013_auto_20201204_0613"),
        ("mocca_screening", "0014_auto_20201204_1949"),
        ("mocca_screening", "0015_auto_20201204_2339"),
        ("mocca_screening", "0016_auto_20201204_2347"),
        ("mocca_screening", "0017_auto_20201205_0401"),
        ("mocca_screening", "0018_auto_20201205_0424"),
        ("mocca_screening", "0019_auto_20201205_1834"),
        ("mocca_screening", "0020_auto_20201206_2110"),
        ("mocca_screening", "0021_auto_20201206_2122"),
        ("mocca_screening", "0022_auto_20201206_2140"),
        ("mocca_screening", "0023_auto_20201206_2205"),
        ("mocca_screening", "0024_auto_20201206_2352"),
    ]

    initial = True

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("mocca_lists", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="MoccaRegister",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(default=edc_utils.date.get_utcnow),
                ),
                (
                    "mocca_screening_identifier",
                    models.CharField(
                        blank=True,
                        help_text="If known",
                        max_length=15,
                        null=True,
                        verbose_name="MOCCA (original) screening identifier",
                    ),
                ),
                (
                    "mocca_study_identifier",
                    models.CharField(
                        help_text="Format must match original identifier. e.g. 12-3456 for UG, 123456 for TZ",
                        max_length=25,
                        validators=[
                            django.core.validators.RegexValidator(
                                "0[0-9]{1}\\-0[0-9]{3}|[0-9]{6}",
                                "Invalid format. Expected 12-3456 for UG, 123456 for TZ",
                            )
                        ],
                        verbose_name="MOCCA (original) study identifier",
                    ),
                ),
                (
                    "mocca_country",
                    models.CharField(
                        choices=[("uganda", "Uganda"), ("tanzania", "Tanzania")],
                        max_length=25,
                    ),
                ),
                (
                    "first_name",
                    django_crypto_fields.fields.firstname_field.FirstnameField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                    ),
                ),
                (
                    "last_name",
                    django_crypto_fields.fields.lastname_field.LastnameField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                    ),
                ),
                (
                    "initials",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text="Use UPPERCASE letters only. May be 2 or 3 letters. (Encryption: RSA local)",
                        max_length=71,
                        validators=[
                            django.core.validators.RegexValidator(
                                "[A-Z]{1,3}", "Invalid format"
                            ),
                            django.core.validators.MinLengthValidator(2),
                            django.core.validators.MaxLengthValidator(3),
                        ],
                    ),
                ),
                (
                    "gender",
                    models.CharField(choices=[("M", "Male"), ("F", "Female")], max_length=10),
                ),
                (
                    "age_in_years",
                    models.IntegerField(
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(18),
                            django.core.validators.MaxValueValidator(110),
                        ],
                    ),
                ),
                (
                    "birth_year",
                    models.IntegerField(
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(1900),
                            django.core.validators.MaxValueValidator(2002),
                        ],
                    ),
                ),
                ("dob", models.DateField(blank=True, null=True)),
                (
                    "survival_status",
                    models.CharField(
                        choices=[
                            ("alive", "Alive"),
                            ("dead", "Deceased"),
                            ("unknown", "Unknown"),
                        ],
                        default="unknown",
                        max_length=25,
                    ),
                ),
                (
                    "mocca_site",
                    models.ForeignKey(
                        limit_choices_to=mocca_screening.mocca_original_sites.get_mocca_site_limited_to,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="mocca_lists.MoccaOriginalSites",
                    ),
                ),
            ],
            options={
                "verbose_name": "MOCCA Patient Register",
                "verbose_name_plural": "MOCCA Patient Register",
                "ordering": ["mocca_country", "mocca_site"],
            },
        ),
        migrations.CreateModel(
            name="SubjectScreening",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50)),
                (
                    "subject_identifier_as_pk",
                    models.UUIDField(default=uuid.uuid4, editable=False),
                ),
                (
                    "subject_identifier_aka",
                    models.CharField(
                        editable=False,
                        help_text="track a previously allocated identifier.",
                        max_length=50,
                        null=True,
                        verbose_name="Subject Identifier a.k.a",
                    ),
                ),
                (
                    "slug",
                    models.CharField(
                        db_index=True,
                        default="",
                        editable=False,
                        help_text="a field used for quick search",
                        max_length=250,
                        null=True,
                    ),
                ),
                (
                    "reference",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        unique=True,
                        verbose_name="Reference",
                    ),
                ),
                (
                    "screening_identifier",
                    models.CharField(
                        blank=True,
                        editable=False,
                        max_length=50,
                        unique=True,
                        verbose_name="Screening ID",
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        help_text="Date and time of report.",
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "gender",
                    models.CharField(choices=[("M", "Male"), ("F", "Female")], max_length=10),
                ),
                (
                    "age_in_years",
                    models.IntegerField(
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(110),
                        ]
                    ),
                ),
                (
                    "consent_ability",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        verbose_name="Participant or legal guardian/representative able and willing to give informed consent.",
                    ),
                ),
                (
                    "unsuitable_for_study",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        default="No",
                        help_text="If YES, patient NOT eligible, please give reason below.",
                        max_length=5,
                        verbose_name="Is there any other reason the patient is deemed to not be suitable for the study?",
                    ),
                ),
                (
                    "reasons_unsuitable",
                    models.TextField(
                        blank=True,
                        max_length=150,
                        null=True,
                        verbose_name="Reason not suitable for the study",
                    ),
                ),
                (
                    "unsuitable_agreed",
                    models.CharField(
                        choices=[
                            ("Yes", "Yes"),
                            ("No", "No"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=5,
                        verbose_name="Does the study coordinator agree that the patient is not suitable for the study?",
                    ),
                ),
                ("eligible", models.BooleanField(default=False, editable=False)),
                (
                    "reasons_ineligible",
                    models.TextField(
                        editable=False,
                        max_length=150,
                        null=True,
                        verbose_name="Reason not eligible",
                    ),
                ),
                (
                    "eligibility_datetime",
                    models.DateTimeField(
                        editable=False,
                        help_text="Date and time eligibility was determined",
                        null=True,
                    ),
                ),
                ("consented", models.BooleanField(default=False, editable=False)),
                ("refused", models.BooleanField(default=False, editable=False)),
                (
                    "screening_consent",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=15,
                        verbose_name="Has the subject given his/her verbal consent to be screened for the MOCCA Extension trial?",
                    ),
                ),
                (
                    "mocca_participant",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        verbose_name="Was the patient enrolled to the original MOCCA study?",
                    ),
                ),
                (
                    "mocca_study_identifier",
                    models.CharField(
                        help_text="Format must match original identifier. e.g. 12-3456 for UG, 123456 for TZ",
                        max_length=25,
                        validators=[
                            django.core.validators.RegexValidator(
                                "0[0-9]{1}\\-0[0-9]{3}|[0-9]{6}",
                                "Invalid format. Expected 12-3456 for UG, 123456 for TZ",
                            )
                        ],
                        verbose_name="Original MOCCA study identifier",
                    ),
                ),
                (
                    "initials",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text="Use UPPERCASE letters only. May be 2 or 3 letters. Use `F`irst`L`ast or `L`ast`F`irst depending on the country custom.  (Encryption: RSA local)",
                        max_length=71,
                        validators=[
                            django.core.validators.RegexValidator(
                                "[A-Z]{1,3}", "Invalid format"
                            ),
                            django.core.validators.MinLengthValidator(2),
                            django.core.validators.MaxLengthValidator(3),
                        ],
                    ),
                ),
                (
                    "birth_year",
                    models.IntegerField(
                        validators=[
                            django.core.validators.MinValueValidator(1900),
                            django.core.validators.MaxValueValidator(2002),
                        ]
                    ),
                ),
                (
                    "mocca_register",
                    models.OneToOneField(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="mocca_screening.MoccaRegister",
                    ),
                ),
                (
                    "mocca_site",
                    models.ForeignKey(
                        limit_choices_to=mocca_screening.mocca_original_sites.get_mocca_site_limited_to,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="mocca_lists.MoccaOriginalSites",
                        verbose_name="Original MOCCA site",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
            ],
            options={
                "verbose_name": "Subject Screening",
                "verbose_name_plural": "Subject Screening",
            },
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                (
                    "objects",
                    edc_screening.model_mixins.screening_fields_model_mixin.ScreeningManager(),
                ),
            ],
        ),
        migrations.CreateModel(
            name="SubjectRefusal",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("screening_identifier", models.CharField(max_length=50, unique=True)),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "reason",
                    models.CharField(
                        choices=[
                            ("unwilling_to_say", "I am unwilling to say"),
                            ("dont_have_time", "I don't have time"),
                            ("stigma", "I am worried about stigma"),
                            ("must_consult_spouse", "I need to consult my spouse"),
                            (
                                "dont_want_medication",
                                "I don't want to take any more medication",
                            ),
                            ("dont_want_to_join", "I don't want to take part"),
                            (
                                "need_to_think_about_it",
                                "I haven't had a chance to think about it",
                            ),
                            ("moving", "I am moving to another area"),
                            ("OTHER", "Other, please specify"),
                        ],
                        max_length=25,
                        verbose_name="Reason for refusal to join",
                    ),
                ),
                (
                    "other_reason",
                    edc_model.models.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other, please specify ...",
                    ),
                ),
                (
                    "comment",
                    models.TextField(
                        blank=True, null=True, verbose_name="Additional Comments"
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
            ],
            options={
                "verbose_name": "Subject Refusal",
                "verbose_name_plural": "Subject Refusals",
                "ordering": ("-modified", "-created"),
                "get_latest_by": "modified",
                "abstract": False,
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
            },
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
            ],
        ),
        migrations.CreateModel(
            name="HistoricalSubjectScreening",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50)),
                (
                    "subject_identifier_as_pk",
                    models.UUIDField(default=uuid.uuid4, editable=False),
                ),
                (
                    "subject_identifier_aka",
                    models.CharField(
                        editable=False,
                        help_text="track a previously allocated identifier.",
                        max_length=50,
                        null=True,
                        verbose_name="Subject Identifier a.k.a",
                    ),
                ),
                (
                    "slug",
                    models.CharField(
                        db_index=True,
                        default="",
                        editable=False,
                        help_text="a field used for quick search",
                        max_length=250,
                        null=True,
                    ),
                ),
                (
                    "reference",
                    models.UUIDField(
                        db_index=True,
                        default=uuid.uuid4,
                        editable=False,
                        verbose_name="Reference",
                    ),
                ),
                (
                    "screening_identifier",
                    models.CharField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        max_length=50,
                        verbose_name="Screening ID",
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        help_text="Date and time of report.",
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "gender",
                    models.CharField(choices=[("M", "Male"), ("F", "Female")], max_length=10),
                ),
                (
                    "age_in_years",
                    models.IntegerField(
                        validators=[
                            django.core.validators.MinValueValidator(0),
                            django.core.validators.MaxValueValidator(110),
                        ]
                    ),
                ),
                (
                    "consent_ability",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        verbose_name="Participant or legal guardian/representative able and willing to give informed consent.",
                    ),
                ),
                (
                    "unsuitable_for_study",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        default="No",
                        help_text="If YES, patient NOT eligible, please give reason below.",
                        max_length=5,
                        verbose_name="Is there any other reason the patient is deemed to not be suitable for the study?",
                    ),
                ),
                (
                    "reasons_unsuitable",
                    models.TextField(
                        blank=True,
                        max_length=150,
                        null=True,
                        verbose_name="Reason not suitable for the study",
                    ),
                ),
                (
                    "unsuitable_agreed",
                    models.CharField(
                        choices=[
                            ("Yes", "Yes"),
                            ("No", "No"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=5,
                        verbose_name="Does the study coordinator agree that the patient is not suitable for the study?",
                    ),
                ),
                ("eligible", models.BooleanField(default=False, editable=False)),
                (
                    "reasons_ineligible",
                    models.TextField(
                        editable=False,
                        max_length=150,
                        null=True,
                        verbose_name="Reason not eligible",
                    ),
                ),
                (
                    "eligibility_datetime",
                    models.DateTimeField(
                        editable=False,
                        help_text="Date and time eligibility was determined",
                        null=True,
                    ),
                ),
                ("consented", models.BooleanField(default=False, editable=False)),
                ("refused", models.BooleanField(default=False, editable=False)),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "screening_consent",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=15,
                        verbose_name="Has the subject given his/her verbal consent to be screened for the MOCCA Extension trial?",
                    ),
                ),
                (
                    "mocca_participant",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        verbose_name="Was the patient enrolled to the original MOCCA study?",
                    ),
                ),
                (
                    "mocca_study_identifier",
                    models.CharField(
                        help_text="Format must match original identifier. e.g. 12-3456 for UG, 123456 for TZ",
                        max_length=25,
                        validators=[
                            django.core.validators.RegexValidator(
                                "0[0-9]{1}\\-0[0-9]{3}|[0-9]{6}",
                                "Invalid format. Expected 12-3456 for UG, 123456 for TZ",
                            )
                        ],
                        verbose_name="Original MOCCA study identifier",
                    ),
                ),
                (
                    "initials",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text="Use UPPERCASE letters only. May be 2 or 3 letters. Use `F`irst`L`ast or `L`ast`F`irst depending on the country custom.  (Encryption: RSA local)",
                        max_length=71,
                        validators=[
                            django.core.validators.RegexValidator(
                                "[A-Z]{1,3}", "Invalid format"
                            ),
                            django.core.validators.MinLengthValidator(2),
                            django.core.validators.MaxLengthValidator(3),
                        ],
                    ),
                ),
                (
                    "birth_year",
                    models.IntegerField(
                        validators=[
                            django.core.validators.MinValueValidator(1900),
                            django.core.validators.MaxValueValidator(2002),
                        ]
                    ),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "mocca_register",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="mocca_screening.MoccaRegister",
                    ),
                ),
                (
                    "mocca_site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        limit_choices_to=mocca_screening.mocca_original_sites.get_mocca_site_limited_to,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="mocca_lists.MoccaOriginalSites",
                        verbose_name="Original MOCCA site",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Subject Screening",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="HistoricalSubjectRefusal",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "screening_identifier",
                    models.CharField(db_index=True, max_length=50),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        verbose_name="Report Date and Time",
                    ),
                ),
                (
                    "reason",
                    models.CharField(
                        choices=[
                            ("unwilling_to_say", "I am unwilling to say"),
                            ("dont_have_time", "I don't have time"),
                            ("stigma", "I am worried about stigma"),
                            ("must_consult_spouse", "I need to consult my spouse"),
                            (
                                "dont_want_medication",
                                "I don't want to take any more medication",
                            ),
                            ("dont_want_to_join", "I don't want to take part"),
                            (
                                "need_to_think_about_it",
                                "I haven't had a chance to think about it",
                            ),
                            ("moving", "I am moving to another area"),
                            ("OTHER", "Other, please specify"),
                        ],
                        max_length=25,
                        verbose_name="Reason for refusal to join",
                    ),
                ),
                (
                    "other_reason",
                    edc_model.models.fields.other_charfield.OtherCharField(
                        blank=True,
                        max_length=35,
                        null=True,
                        verbose_name="If other, please specify ...",
                    ),
                ),
                (
                    "comment",
                    models.TextField(
                        blank=True, null=True, verbose_name="Additional Comments"
                    ),
                ),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Subject Refusal",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="HistoricalDailyClosingLog",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "log_date",
                    models.DateField(
                        default=edc_utils.date.get_utcnow, verbose_name="Clinic date"
                    ),
                ),
                (
                    "clinic_services",
                    models.CharField(
                        choices=[
                            (
                                "integrated",
                                "Integrated care day (HIV, Diabetes, Hypertension)",
                            ),
                            ("ncd", "NCD day (Diabetes + Hypertension)"),
                            ("HIV", "HIV only day"),
                            ("diabetes", "Diabetes only day"),
                            ("hypertension", "Hypertension only day"),
                        ],
                        max_length=25,
                        verbose_name="Which services are being offered at the clinic today?",
                    ),
                ),
                (
                    "attended",
                    models.IntegerField(
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Total number of patients who attended the clinic today",
                    ),
                ),
                (
                    "selection_method",
                    models.CharField(
                        choices=[
                            ("random_sampling", "Random sampling"),
                            ("systematic", "Systematically selected"),
                            ("sequential", "Sequentially selected"),
                            ("purposively_selected", "Purposively selected"),
                        ],
                        max_length=25,
                        verbose_name="How were patients selected to be approached?",
                    ),
                ),
                (
                    "approached",
                    models.IntegerField(
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Of those who attended, how many were approached by the study team",
                    ),
                ),
                (
                    "agreed_to_screen",
                    models.IntegerField(
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Of those approached, how many agreed to be screened",
                    ),
                ),
                (
                    "comment",
                    models.TextField(
                        blank=True, null=True, verbose_name="Additional Comments"
                    ),
                ),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Daily Closing Log",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="DailyClosingLog",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "log_date",
                    models.DateField(
                        default=edc_utils.date.get_utcnow, verbose_name="Clinic date"
                    ),
                ),
                (
                    "clinic_services",
                    models.CharField(
                        choices=[
                            (
                                "integrated",
                                "Integrated care day (HIV, Diabetes, Hypertension)",
                            ),
                            ("ncd", "NCD day (Diabetes + Hypertension)"),
                            ("HIV", "HIV only day"),
                            ("diabetes", "Diabetes only day"),
                            ("hypertension", "Hypertension only day"),
                        ],
                        max_length=25,
                        verbose_name="Which services are being offered at the clinic today?",
                    ),
                ),
                (
                    "attended",
                    models.IntegerField(
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Total number of patients who attended the clinic today",
                    ),
                ),
                (
                    "selection_method",
                    models.CharField(
                        choices=[
                            ("random_sampling", "Random sampling"),
                            ("systematic", "Systematically selected"),
                            ("sequential", "Sequentially selected"),
                            ("purposively_selected", "Purposively selected"),
                        ],
                        max_length=25,
                        verbose_name="How were patients selected to be approached?",
                    ),
                ),
                (
                    "approached",
                    models.IntegerField(
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Of those who attended, how many were approached by the study team",
                    ),
                ),
                (
                    "agreed_to_screen",
                    models.IntegerField(
                        validators=[django.core.validators.MinValueValidator(0)],
                        verbose_name="Of those approached, how many agreed to be screened",
                    ),
                ),
                (
                    "comment",
                    models.TextField(
                        blank=True, null=True, verbose_name="Additional Comments"
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
            ],
            options={
                "verbose_name": "Daily Closing Log",
                "verbose_name_plural": "Daily Closing Logs",
            },
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
            ],
        ),
        migrations.AddIndex(
            model_name="moccaregister",
            index=models.Index(
                fields=["mocca_country", "mocca_site"],
                name="mocca_scree_mocca_c_7f1635_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="moccaregister",
            index=models.Index(
                fields=["mocca_study_identifier", "initials", "gender"],
                name="mocca_scree_mocca_s_a92e02_idx",
            ),
        ),
        migrations.AddConstraint(
            model_name="moccaregister",
            constraint=models.UniqueConstraint(
                fields=("mocca_screening_identifier",),
                name="unique_mocca_screening_identifier",
            ),
        ),
        migrations.AddConstraint(
            model_name="moccaregister",
            constraint=models.UniqueConstraint(
                fields=("mocca_study_identifier",), name="unique_mocca_study_identifier"
            ),
        ),
        migrations.AddConstraint(
            model_name="moccaregister",
            constraint=models.UniqueConstraint(
                fields=("first_name", "last_name"), name="unique_first_name__last_name"
            ),
        ),
        migrations.AddConstraint(
            model_name="dailyclosinglog",
            constraint=models.UniqueConstraint(
                fields=("log_date", "site"), name="unique_date_for_site"
            ),
        ),
        migrations.AlterModelManagers(
            name="moccaregister",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", mocca_screening.models.mocca_register.Manager()),
            ],
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="site",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.Site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="mocca_study_identifier",
            field=models.CharField(
                db_index=True,
                help_text="Format must match original identifier. e.g. 12-3456 for UG, 123456 for TZ",
                max_length=25,
                validators=[
                    django.core.validators.RegexValidator(
                        "0[0-9]{1}\\-0[0-9]{3}|[0-9]{6}",
                        "Invalid format. Expected 12-3456 for UG, 123456 for TZ",
                    )
                ],
                verbose_name="Original MOCCA study identifier",
            ),
        ),
        migrations.AlterField(
            model_name="moccaregister",
            name="initials",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text="Use UPPERCASE letters only. May be 2 or 3 letters. (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[
                    django.core.validators.RegexValidator("[A-Z]{1,3}", "Invalid format"),
                    django.core.validators.MinLengthValidator(2),
                    django.core.validators.MaxLengthValidator(3),
                ],
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="mocca_study_identifier",
            field=models.CharField(
                help_text="Format must match original identifier. e.g. 12-3456 for UG, 123456 for TZ",
                max_length=25,
                unique=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "0[0-9]{1}\\-0[0-9]{3}|[0-9]{6}",
                        "Invalid format. Expected 12-3456 for UG, 123456 for TZ",
                    )
                ],
                verbose_name="Original MOCCA study identifier",
            ),
        ),
        migrations.AlterField(
            model_name="moccaregister",
            name="gender",
            field=models.CharField(
                choices=[("M", "Male"), ("F", "Female")], max_length=10, null=True
            ),
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="screening_identifier",
            field=models.CharField(
                max_length=15,
                null=True,
                verbose_name="MOCCA (ext) screening identifier",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="mocca_register",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"screening_identifier__isnull": True},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="mocca_screening.MoccaRegister",
                verbose_name="Select one from the MOCCA (original) register",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="mocca_register",
            field=models.OneToOneField(
                limit_choices_to={"screening_identifier__isnull": True},
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="mocca_screening.MoccaRegister",
                verbose_name="Select one from the MOCCA (original) register",
            ),
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="contact_attempts",
            field=models.IntegerField(default=0),
        ),
        migrations.CreateModel(
            name="MoccaRegisterContact",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(default=edc_utils.date.get_utcnow),
                ),
                (
                    "contacted",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        default="No",
                        max_length=15,
                    ),
                ),
                (
                    "survival_status",
                    models.CharField(
                        choices=[
                            ("alive", "Alive"),
                            ("dead", "Deceased"),
                            ("unknown", "Unknown"),
                        ],
                        max_length=15,
                        null=True,
                    ),
                ),
                (
                    "willing_to_attend",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")], max_length=15, null=True
                    ),
                ),
                (
                    "call_again",
                    models.CharField(choices=[("Yes", "Yes"), ("No", "No")], max_length=15),
                ),
                (
                    "mocca_register",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="mocca_screening.MoccaRegister",
                    ),
                ),
            ],
            options={
                "verbose_name": "MOCCA Patient Register Contact",
                "verbose_name_plural": "MOCCA Patient Register Contacts",
            },
        ),
        migrations.AlterModelManagers(
            name="moccaregistercontact",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", mocca_screening.models.mocca_register_contact.Manager()),
            ],
        ),
        migrations.AddField(
            model_name="moccaregistercontact",
            name="site",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.Site",
            ),
        ),
        migrations.RenameField(
            model_name="moccaregistercontact",
            old_name="contacted",
            new_name="answered",
        ),
        migrations.AlterField(
            model_name="moccaregistercontact",
            name="call_again",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Try again?",
            ),
        ),
        migrations.AlterField(
            model_name="moccaregistercontact",
            name="survival_status",
            field=models.CharField(
                choices=[
                    ("alive", "Alive"),
                    ("dead", "Deceased"),
                    ("unknown", "Unknown"),
                ],
                default="unknown",
                max_length=15,
            ),
        ),
        migrations.AlterField(
            model_name="moccaregistercontact",
            name="willing_to_attend",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("unknown", "Unknown")],
                default="unknown",
                max_length=15,
            ),
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="call",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")], default="Yes", max_length=15
            ),
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="notes",
            field=django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                blank=True,
                help_text=" (Encryption: AES local)",
                max_length=71,
                null=True,
            ),
        ),
        migrations.CreateModel(
            name="HistoricalMoccaRegisterContact",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(default=edc_utils.date.get_utcnow),
                ),
                (
                    "answered",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        default="No",
                        max_length=15,
                    ),
                ),
                (
                    "survival_status",
                    models.CharField(
                        choices=[
                            ("alive", "Alive"),
                            ("dead", "Deceased"),
                            ("unknown", "Unknown"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=15,
                    ),
                ),
                (
                    "willing_to_attend",
                    models.CharField(
                        choices=[
                            ("Yes", "Yes"),
                            ("No", "No"),
                            ("not_sure", "Not sure"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=15,
                    ),
                ),
                (
                    "call_again",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=15,
                        verbose_name="Try again?",
                    ),
                ),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "mocca_register",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="mocca_screening.MoccaRegister",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
                (
                    "respondent",
                    models.CharField(
                        choices=[
                            ("patient", "Patient"),
                            ("family", "Family"),
                            ("friend", "friend"),
                            ("other", "other"),
                            ("N/A", "Not applicable"),
                        ],
                        default="N/A",
                        max_length=15,
                    ),
                ),
            ],
            options={
                "verbose_name": "historical MOCCA Patient Register Contact",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.AddField(
            model_name="moccaregistercontact",
            name="respondent",
            field=models.CharField(
                choices=[
                    ("patient", "Patient"),
                    ("family", "Family"),
                    ("friend", "friend"),
                    ("other", "other"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
            ),
        ),
        migrations.AlterField(
            model_name="moccaregistercontact",
            name="survival_status",
            field=models.CharField(
                choices=[
                    ("alive", "Alive"),
                    ("dead", "Deceased"),
                    ("unknown", "Unknown"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
            ),
        ),
        migrations.AlterField(
            model_name="moccaregistercontact",
            name="willing_to_attend",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("not_sure", "Not sure"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
            ),
        ),
        migrations.CreateModel(
            name="HistoricalMoccaRegister",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(default=edc_utils.date.get_utcnow),
                ),
                (
                    "mocca_screening_identifier",
                    models.CharField(
                        blank=True,
                        help_text="If known",
                        max_length=15,
                        null=True,
                        verbose_name="MOCCA (original) screening identifier",
                    ),
                ),
                (
                    "mocca_study_identifier",
                    models.CharField(
                        help_text="Format must match original identifier. e.g. 12-3456 for UG, 123456 for TZ",
                        max_length=25,
                        validators=[
                            django.core.validators.RegexValidator(
                                "0[0-9]{1}\\-0[0-9]{3}|[0-9]{6}",
                                "Invalid format. Expected 12-3456 for UG, 123456 for TZ",
                            )
                        ],
                        verbose_name="MOCCA (original) study identifier",
                    ),
                ),
                (
                    "mocca_country",
                    models.CharField(
                        choices=[("uganda", "Uganda"), ("tanzania", "Tanzania")],
                        max_length=25,
                    ),
                ),
                (
                    "first_name",
                    django_crypto_fields.fields.firstname_field.FirstnameField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                    ),
                ),
                (
                    "last_name",
                    django_crypto_fields.fields.lastname_field.LastnameField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                    ),
                ),
                (
                    "initials",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text="Use UPPERCASE letters only. May be 2 or 3 letters. (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[
                            django.core.validators.RegexValidator(
                                "[A-Z]{1,3}", "Invalid format"
                            ),
                            django.core.validators.MinLengthValidator(2),
                            django.core.validators.MaxLengthValidator(3),
                        ],
                    ),
                ),
                (
                    "gender",
                    models.CharField(
                        choices=[("M", "Male"), ("F", "Female")],
                        max_length=10,
                        null=True,
                    ),
                ),
                (
                    "age_in_years",
                    models.IntegerField(
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(18),
                            django.core.validators.MaxValueValidator(110),
                        ],
                    ),
                ),
                (
                    "birth_year",
                    models.IntegerField(
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(1900),
                            django.core.validators.MaxValueValidator(2002),
                        ],
                    ),
                ),
                ("dob", models.DateField(blank=True, null=True)),
                (
                    "survival_status",
                    models.CharField(
                        choices=[
                            ("alive", "Alive"),
                            ("dead", "Deceased"),
                            ("unknown", "Unknown"),
                        ],
                        default="unknown",
                        max_length=25,
                    ),
                ),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "mocca_site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        limit_choices_to=mocca_screening.mocca_original_sites.get_mocca_site_limited_to,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="mocca_lists.MoccaOriginalSites",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
                (
                    "screening_identifier",
                    models.CharField(
                        max_length=15,
                        null=True,
                        verbose_name="MOCCA (ext) screening identifier",
                    ),
                ),
                ("contact_attempts", models.IntegerField(default=0)),
                (
                    "call",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        default="Yes",
                        max_length=15,
                    ),
                ),
                (
                    "notes",
                    django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                        blank=True,
                        help_text=" (Encryption: AES local)",
                        max_length=71,
                        null=True,
                    ),
                ),
                ("date_last_called", models.DateField(null=True)),
            ],
            options={
                "verbose_name": "historical MOCCA Patient Register",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="date_last_called",
            field=models.DateField(null=True),
        ),
        migrations.RemoveConstraint(
            model_name="moccaregister",
            name="unique_first_name__last_name",
        ),
        migrations.AlterModelOptions(
            name="moccaregistercontact",
            options={
                "ordering": ["report_datetime"],
                "verbose_name": "MOCCA Patient Register Contact",
                "verbose_name_plural": "MOCCA Patient Register Contacts",
            },
        ),
        migrations.AlterField(
            model_name="historicalmoccaregister",
            name="call",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=15,
                verbose_name="Call?",
            ),
        ),
        migrations.AlterField(
            model_name="moccaregister",
            name="call",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=15,
                verbose_name="Call?",
            ),
        ),
        migrations.AddField(
            model_name="historicalmoccaregister",
            name="tel_one",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                verbose_name="Tel/Mobile(1)",
            ),
        ),
        migrations.AddField(
            model_name="historicalmoccaregister",
            name="tel_three",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                verbose_name="Tel/Mobile(3)",
            ),
        ),
        migrations.AddField(
            model_name="historicalmoccaregister",
            name="tel_two",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                verbose_name="Tel/Mobile(2)",
            ),
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="tel_one",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                verbose_name="Tel/Mobile(1)",
            ),
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="tel_three",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                verbose_name="Tel/Mobile(3)",
            ),
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="tel_two",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                verbose_name="Tel/Mobile(2)",
            ),
        ),
        migrations.AddField(
            model_name="historicalmoccaregister",
            name="best_tel",
            field=models.CharField(
                blank=True,
                choices=[
                    ("tel_1", "Tel/Mobile(1)"),
                    ("tel_2", "Tel/Mobile(2)"),
                    ("tel_3", "Tel/Mobile(3)"),
                ],
                help_text="If any, select the best telephone/mobile from above",
                max_length=15,
                null=True,
                verbose_name="Prefered Telephone / Mobile",
            ),
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="best_tel",
            field=models.CharField(
                blank=True,
                choices=[
                    ("tel_1", "Tel/Mobile(1)"),
                    ("tel_2", "Tel/Mobile(2)"),
                    ("tel_3", "Tel/Mobile(3)"),
                ],
                help_text="If any, select the best telephone/mobile from above",
                max_length=15,
                null=True,
                verbose_name="Prefered Telephone / Mobile",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmoccaregistercontact",
            name="answered",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")], max_length=15, null=True
            ),
        ),
        migrations.AlterField(
            model_name="moccaregistercontact",
            name="answered",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")], max_length=15, null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalmoccaregistercontact",
            name="call_again",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Call again?",
            ),
        ),
        migrations.AlterField(
            model_name="moccaregistercontact",
            name="call_again",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Call again?",
            ),
        ),
        migrations.AddField(
            model_name="historicalmoccaregister",
            name="next_appt_date",
            field=models.DateField(
                blank=True, help_text="auto-updated", null=True, verbose_name="Appt"
            ),
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="next_appt_date",
            field=models.DateField(
                blank=True, help_text="auto-updated", null=True, verbose_name="Appt"
            ),
        ),
        migrations.AlterField(
            model_name="historicalmoccaregister",
            name="contact_attempts",
            field=models.IntegerField(default=0, help_text="auto-updated"),
        ),
        migrations.AlterField(
            model_name="historicalmoccaregister",
            name="date_last_called",
            field=models.DateField(help_text="auto-updated", null=True),
        ),
        migrations.AlterField(
            model_name="historicalmoccaregister",
            name="notes",
            field=django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                blank=True,
                help_text=" (Encryption: AES local)",
                max_length=71,
                null=True,
                verbose_name="General notes",
            ),
        ),
        migrations.AddField(
            model_name="historicalmoccaregistercontact",
            name="comment",
            field=django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                blank=True,
                help_text=" (Encryption: AES local)",
                max_length=71,
                null=True,
                verbose_name="Note",
            ),
        ),
        migrations.AddField(
            model_name="historicalmoccaregistercontact",
            name="next_appt_date",
            field=models.DateField(blank=True, null=True, verbose_name="Next Appt."),
        ),
        migrations.AlterField(
            model_name="moccaregister",
            name="contact_attempts",
            field=models.IntegerField(default=0, help_text="auto-updated"),
        ),
        migrations.AlterField(
            model_name="moccaregister",
            name="date_last_called",
            field=models.DateField(help_text="auto-updated", null=True),
        ),
        migrations.AlterField(
            model_name="moccaregister",
            name="notes",
            field=django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                blank=True,
                help_text=" (Encryption: AES local)",
                max_length=71,
                null=True,
                verbose_name="General notes",
            ),
        ),
        migrations.AddField(
            model_name="moccaregistercontact",
            name="comment",
            field=django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                blank=True,
                help_text=" (Encryption: AES local)",
                max_length=71,
                null=True,
                verbose_name="Note",
            ),
        ),
        migrations.AddField(
            model_name="moccaregistercontact",
            name="next_appt_date",
            field=models.DateField(blank=True, null=True, verbose_name="Next Appt."),
        ),
        migrations.AddField(
            model_name="historicalmoccaregistercontact",
            name="death_date",
            field=models.DateField(blank=True, null=True, verbose_name="Date of death"),
        ),
        migrations.AddField(
            model_name="moccaregistercontact",
            name="death_date",
            field=models.DateField(blank=True, null=True, verbose_name="Date of death"),
        ),
        migrations.AddField(
            model_name="historicalmoccaregistercontact",
            name="icc",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Either at this facility or elsewhere",
                max_length=25,
                null=True,
                verbose_name="Does the patient currently receive regular integrated care",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="care_not_in_reason",
            field=models.CharField(
                blank=True,
                max_length=25,
                null=True,
                verbose_name="If not in care, why?",
            ),
        ),
        migrations.AddField(
            model_name="moccaregistercontact",
            name="icc",
            field=models.CharField(
                blank=True,
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="Either at this facility or elsewhere",
                max_length=25,
                null=True,
                verbose_name="Does the patient currently receive regular integrated care",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="care_not_in_reason",
            field=models.CharField(
                blank=True,
                max_length=25,
                null=True,
                verbose_name="If not in care, why?",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="care",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("unknown", "Unknown")],
                default="-",
                max_length=25,
                verbose_name="Is the patient in care?",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="care_facility_location",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="Either integrated care or vertical care",
                max_length=25,
                verbose_name="Does the patient receive care in this facility",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="mocca_participant",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=25,
                verbose_name="Was the patient enrolled to the original MOCCA study?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="mocca_study_identifier",
            field=models.CharField(
                db_index=True,
                help_text="Format must match original identifier. e.g. 12-3456 for UG, 123456 for TZ",
                max_length=25,
                validators=[
                    django.core.validators.RegexValidator(
                        "0[0-9]{1}\\-0[0-9]{3}|[0-9]{6}",
                        "Invalid format. Expected 12-3456 for UG, 123456 for TZ",
                    )
                ],
                verbose_name="MOCCA (original) study identifier",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="care",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("unknown", "Unknown")],
                default="-",
                max_length=25,
                verbose_name="Is the patient in care?",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="care_facility_location",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="Either integrated care or vertical care",
                max_length=25,
                verbose_name="Does the patient receive care in this facility",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="mocca_participant",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="Yes",
                max_length=25,
                verbose_name="Was the patient enrolled to the original MOCCA study?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="mocca_study_identifier",
            field=models.CharField(
                help_text="Format must match original identifier. e.g. 12-3456 for UG, 123456 for TZ",
                max_length=25,
                unique=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "0[0-9]{1}\\-0[0-9]{3}|[0-9]{6}",
                        "Invalid format. Expected 12-3456 for UG, 123456 for TZ",
                    )
                ],
                verbose_name="MOCCA (original) study identifier",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="care_comment",
            field=models.TextField(
                blank=True,
                null=True,
                verbose_name="Additional comments relevant to this patient's care",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="icc_not_in_reason",
            field=models.CharField(
                choices=[
                    (
                        "icc_not_available",
                        "ICC not available (or closed) in this facility",
                    ),
                    ("moved", "Moved out of area"),
                    (
                        "dont_want",
                        "Personally chose not to continue with integrated care",
                    ),
                    (
                        "advised_to_vertical",
                        "Healthcare staff asked patient to return to vertical care",
                    ),
                    ("referred_out", "Referred to another facility without an ICC"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="If not integrated care, why not?",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="care_comment",
            field=models.TextField(
                blank=True,
                null=True,
                verbose_name="Additional comments relevant to this patient's care",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="icc_not_in_reason",
            field=models.CharField(
                choices=[
                    (
                        "icc_not_available",
                        "ICC not available (or closed) in this facility",
                    ),
                    ("moved", "Moved out of area"),
                    (
                        "dont_want",
                        "Personally chose not to continue with integrated care",
                    ),
                    (
                        "advised_to_vertical",
                        "Healthcare staff asked patient to return to vertical care",
                    ),
                    ("referred_out", "Referred to another facility without an ICC"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="If not integrated care, why not?",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="icc",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="Either at this facility or elsewhere",
                max_length=25,
                verbose_name="Does the patient <u>currently</u> receive integrated care",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="icc_since_mocca",
            field=models.CharField(
                choices=[
                    ("no_interruption", "Yes, without interruption"),
                    ("some_interruption", "Yes, with some interruption"),
                    (
                        "No",
                        "No, not since completing follow up with the MOCCA (original) trial.",
                    ),
                    ("unknown", "Unknown"),
                ],
                max_length=25,
                verbose_name="Has the patient received integrated care since the leaving the MOCCA (orig) trial until now?",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="icc_since_mocca_comment",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If some interruption in integrated care, please explain",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="mocca_register",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="mocca_screening.MoccaRegister",
                verbose_name="MOCCA (original) register details",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectscreening",
            name="willing_to_consent",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="If Yes, begin the informed consent process.",
                max_length=25,
                verbose_name="Is the patient willing and able to participate in the `MOCCA extension` trial",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="icc",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="Either at this facility or elsewhere",
                max_length=25,
                verbose_name="Does the patient <u>currently</u> receive integrated care",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="icc_since_mocca",
            field=models.CharField(
                choices=[
                    ("no_interruption", "Yes, without interruption"),
                    ("some_interruption", "Yes, with some interruption"),
                    (
                        "No",
                        "No, not since completing follow up with the MOCCA (original) trial.",
                    ),
                    ("unknown", "Unknown"),
                ],
                max_length=25,
                verbose_name="Has the patient received integrated care since the leaving the MOCCA (orig) trial until now?",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="icc_since_mocca_comment",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If some interruption in integrated care, please explain",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="mocca_register",
            field=models.OneToOneField(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="mocca_screening.MoccaRegister",
                verbose_name="MOCCA (original) register details",
            ),
        ),
        migrations.AddField(
            model_name="subjectscreening",
            name="willing_to_consent",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="If Yes, begin the informed consent process.",
                max_length=25,
                verbose_name="Is the patient willing and able to participate in the `MOCCA extension` trial",
            ),
        ),
    ]



import sys
from unittest import skip

from dateutil.relativedelta import relativedelta
from django.apps import apps as django_apps
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.core.management.color import color_style
from django.test import TestCase
from django.test.utils import override_settings
from django.urls.base import reverse
from django.urls.exceptions import NoReverseMatch
from edc_adverse_event.auth_objects import TMG
from edc_appointment.constants import IN_PROGRESS_APPT, SCHEDULED_APPT
from edc_appointment.models import Appointment
from edc_auth.auth_objects import AUDITOR, CLINIC, EVERYONE, PII
from edc_dashboard.url_names import url_names
from edc_export.auth_objects import EXPORT
from edc_lab.auth_objects import LAB
from edc_sites import add_or_update_django_sites, get_sites_by_country
from edc_utils import get_utcnow
from model_bakery import baker

from mocca_screening.models.subject_screening import SubjectScreening

from ..mocca_test_case_mixin import MoccaTestCaseMixin

style = color_style()

User = get_user_model()

app_prefix = "mocca"


def login(testcase, user=None, superuser=None, groups=None):
    user = testcase.user if user is None else user
    superuser = True if superuser is None else superuser
    if not superuser:
        user.is_superuser = False
        user.is_active = True
        user.save()
        for group_name in groups:
            group = Group.objects.get(name=group_name)
            user.groups.add(group)
    return testcase.client.force_login(user or testcase.user)


@skip
@override_settings(SIMPLE_HISTORY_PERMISSIONS_ENABLED=True)
class AdminSiteTest(MoccaTestCaseMixin, TestCase):
    def setUp(self):
        self.user = User.objects.create_superuser("user_login", "u@example.com", "pass")

    def test_ae(self):
        self.login(superuser=False, groups=[EVERYONE, AUDITOR])
        response = self.client.get(reverse("mocca_ae:home_url"))
        self.assertEqual(response.status_code, 200)
        response = self.client.get(reverse("edc_adverse_event:ae_home_url"))
        self.assertEqual(response.status_code, 200)
        response = self.client.get(reverse("edc_adverse_event:tmg_home_url"))
        self.assertEqual(response.status_code, 200)
        response = self.client.get(reverse("edc_data_manager:home_url"))
        self.assertEqual(response.status_code, 200)

    def test_home_everyone(self):
        self.login(superuser=False, groups=[EVERYONE])
        response = self.client.get(reverse("home_url"))
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, "Screening")
        self.assertNotContains(response, "Subjects")
        self.assertNotContains(response, "Specimens")
        self.assertNotContains(response, "Adverse&nbsp;events")
        self.assertNotContains(response, "TMG&nbsp;reports")
        self.assertNotContains(response, "Pharmacy")
        self.assertNotContains(response, "Action&nbsp;items")
        self.assertNotContains(response, "Export&nbsp;data")
        self.assertNotContains(response, "Synchronization")

        self.assertContains(response, "Switch&nbsp;sites")
        self.assertContains(response, "Log&nbsp;out")

    def test_home_auditor(self):
        self.login(superuser=False, groups=[EVERYONE, AUDITOR])
        response = self.client.get(reverse("home_url"))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Screening")
        self.assertContains(response, "Subjects")
        self.assertContains(response, "Specimens")
        self.assertContains(response, "Adverse&nbsp;events")
        self.assertContains(response, "TMG&nbsp;reports")

        self.assertNotContains(response, "Pharmacy")

        self.assertContains(response, "Action&nbsp;items")

        self.assertNotContains(response, "Export&nbsp;data")
        self.assertNotContains(response, "Synchronization")

        self.assertContains(response, "Switch&nbsp;sites")
        self.assertContains(response, "Log&nbsp;out")

    def test_home_clinic(self):
        self.login(superuser=False, groups=[EVERYONE, CLINIC, PII])
        response = self.client.get(reverse("home_url"))
        self.assertContains(response, "Screening")
        self.assertContains(response, "Subjects")
        self.assertContains(response, "Specimens")
        self.assertContains(response, "Adverse&nbsp;events")
        self.assertContains(response, "TMG&nbsp;reports")
        self.assertNotContains(response, "Pharmacy")

        self.assertContains(response, "Action&nbsp;items")

        self.assertNotContains(response, "Export&nbsp;data")
        self.assertNotContains(response, "Synchronization")

        self.assertContains(response, "Switch&nbsp;sites")
        self.assertContains(response, "Log&nbsp;out")

    def test_home_export(self):
        self.login(superuser=False, groups=[EVERYONE, EXPORT])
        response = self.client.get(reverse("home_url"))
        self.assertNotContains(response, "Screening")
        self.assertNotContains(response, "Subjects")
        self.assertNotContains(response, "Specimens")
        self.assertNotContains(response, "Adverse&nbsp;events")
        self.assertNotContains(response, "TMG&nbsp;reports")
        self.assertNotContains(response, "Pharmacy")
        self.assertContains(response, "Action&nbsp;items")
        self.assertContains(response, "Export&nbsp;data")
        self.assertNotContains(response, "Synchronization")
        self.assertContains(response, "Switch&nbsp;sites")
        self.assertContains(response, "Log&nbsp;out")

    def test_home_tmg(self):
        self.login(superuser=False, groups=[EVERYONE, TMG])
        response = self.client.get(reverse("home_url"))

        self.assertContains(response, "Screening")
        self.assertContains(response, "Subjects")
        self.assertNotContains(response, "Specimens")
        self.assertContains(response, "Adverse&nbsp;events")
        self.assertContains(response, "TMG&nbsp;reports")
        self.assertNotContains(response, "Pharmacy")
        self.assertContains(response, "Action&nbsp;items")
        self.assertNotContains(response, "Export&nbsp;data")
        self.assertNotContains(response, "Synchronization")
        self.assertContains(response, "Switch&nbsp;sites")
        self.assertContains(response, "Log&nbsp;out")

    def test_home_lab(self):
        self.login(superuser=False, groups=[EVERYONE, LAB])
        response = self.client.get(reverse("home_url"))

        self.assertContains(response, "Screening")
        self.assertContains(response, "Subjects")
        self.assertContains(response, "Specimens")
        self.assertNotContains(response, "Adverse&nbsp;events")
        self.assertNotContains(response, "TMG&nbsp;reports")
        self.assertNotContains(response, "Pharmacy")
        self.assertNotContains(response, "Action&nbsp;items")
        self.assertNotContains(response, "Export&nbsp;data")
        self.assertNotContains(response, "Synchronization")
        self.assertContains(response, "Switch&nbsp;sites")
        self.assertContains(response, "Log&nbsp;out")

    def test_screening_no_pii(self):
        self.login(superuser=False, groups=[EVERYONE, CLINIC])
        home_page = self.client.get(reverse("home_url"))
        screening_page = home_page.click(description="Screening", index=1)
        self.assertNotIn("Add SubjectScreening", screening_page)

    def test_screening_form(self):
        subject_screening = baker.prepare_recipe(f"{app_prefix}_screening.subjectscreening")
        self.login(superuser=False, groups=[EVERYONE, CLINIC, PII])

        home_page = self.client.get(reverse("home_url"))
        screening_listboard_page = home_page.click(description="Screening", index=1)
        add_screening_page = screening_listboard_page.click(
            description="Add Subject Screening"
        )

        # submit blank form
        response = add_screening_page.form.submit()
        self.assertIn("Please correct the errors below", response)

        # submit completed form
        for field, _ in add_screening_page.form.fields.items():
            try:
                add_screening_page.form[field] = getattr(subject_screening, field)
            except AttributeError:
                pass
        page = add_screening_page.form.submit()

        # redirects back to listboard
        self.assertRedirects(page, reverse("mocca_dashboard:screening_listboard_url"))

        # new screened subject is available
        obj = SubjectScreening.objects.all().last()
        screening_listboard_page = home_page.click(description="Screening", index=1)
        self.assertIn(obj.screening_identifier, screening_listboard_page)

        add_subjectconsent_page = screening_listboard_page.click(
            description="Consent", index=1
        )

        self.assertEqual(add_subjectconsent_page.status_code, 200)

    def test_to_subject_dashboard(self):
        add_or_update_django_sites(apps=django_apps, sites=get_sites_by_country("uganda"))
        self.login(superuser=False, groups=[EVERYONE, CLINIC, PII])
        subject_screening = self.get_subject_screening()
        home_page = self.client.get(reverse("home_url"))
        screening_listboard_page = home_page.click(description="Screening", index=1)
        add_subjectconsent_page = screening_listboard_page.click(
            description="Consent", index=1
        )
        # submit blank form
        response = add_subjectconsent_page.form.submit()
        self.assertIn("Please correct the errors below", response)

        subject_consent = baker.make_recipe(
            "mocca_subject.subjectconsent",
            screening_identifier=subject_screening.screening_identifier,
            dob=(get_utcnow() - relativedelta(years=subject_screening.age_in_years)).date(),
            first_name="Melissa",
            last_name="Rodriguez",
            initials="MR",
            consent_datetime=get_utcnow(),
        )

        home_page = self.client.get(reverse("home_url"), user=self.user, status=200)
        screening_listboard_page = home_page.click(description="Screening", index=1)

        self.assertIn("Dashboard", screening_listboard_page)
        self.assertIn(
            f"subjectscreening_change_{subject_screening.screening_identifier}",
            screening_listboard_page,
        )

        home_page = self.client.get(reverse("home_url"), user=self.user, status=200)
        subject_listboard_page = home_page.click(description="Subjects", index=1)

        self.assertIn(subject_consent.subject_identifier, subject_listboard_page)

        href = reverse(
            "mocca_dashboard:subject_dashboard_url",
            kwargs={"subject_identifier": subject_consent.subject_identifier},
        )
        subject_dashboard_page = subject_listboard_page.click(href=href)

        self.assertEqual(subject_dashboard_page.status_code, 200)

        # on subject_dashboard
        # assert all appointment are showing
        subject_identifier = subject_consent.subject_identifier
        appointments = Appointment.objects.filter(
            subject_identifier=subject_identifier
        ).order_by("appt_datetime")
        for appointment in appointments:
            self.assertIn(appointment.visit_code, subject_dashboard_page)

        # start appointment 1000
        page = subject_dashboard_page.click(linkid="start_btn_1000")
        page.form["appt_status"] = IN_PROGRESS_APPT
        page.form["appt_reason"] = SCHEDULED_APPT
        subject_dashboard_page = page.form.submit()
        self.assertEqual(subject_dashboard_page.status_code, 302)
        self.assertEqual(
            subject_dashboard_page.url,
            f"/subject/subject_dashboard/{subject_identifier}/",
        )

        subject_dashboard_page = self.client.get(
            subject_dashboard_page.url, user=self.user, status=200
        )

        # start visit 1000
        self.assertIn(" Start ", subject_dashboard_page)
        subject_visit_page = subject_dashboard_page.click(
            linkid=(
                f"start_btn_{appointments[0].visit_code}_"
                f"{appointments[0].visit_code_sequence}"
            )
        )
        subject_visit_page.form["info_source"] = "patient"
        subject_dashboard_page = subject_visit_page.form.submit()

        url = f"/subject/subject_dashboard/{subject_identifier}/" f"{str(appointments[0].pk)}/"
        self.assertEqual(subject_dashboard_page.status_code, 302)
        self.assertEqual(subject_dashboard_page.url, url)

        subject_dashboard_page = self.client.get(
            reverse(
                "mocca_dashboard:subject_dashboard_url",
                kwargs=dict(
                    subject_identifier=subject_identifier,
                    appointment=str(appointments[0].id),
                ),
            ),
            user=self.user,
            status=200,
        )

        self.assertIn("CRFs", subject_dashboard_page)
        self.assertIn("Requisitions", subject_dashboard_page)

    def test_follow_urls(self):
        """Follows any url that can be reversed without kwargs."""
        self.login(superuser=False, groups=[EVERYONE, CLINIC, PII])
        for url_name in url_names.registry.values():
            sys.stdout.write(style.MIGRATE_HEADING(f" - '{url_name}' ...\r"))
            try:
                url = reverse(url_name)
            except NoReverseMatch:
                sys.stdout.write(
                    style.ERROR(
                        f" - '{url_name}'. Got `NoReverseMatch` "
                        f"when reversed without kwargs.\n"
                    )
                )
            else:
                response = self.client.get(url)
                if response.status_code != 200:
                    sys.stdout.write(
                        style.ERROR(f" - '{url_name}'. Got `{response.status_code}`\n")
                    )
                else:
                    sys.stdout.write(style.SUCCESS(f" - '{url_name}'->{url}\n"))



from django import forms
from edc_constants.constants import NOT_APPLICABLE, OTHER, STUDY_DEFINED_TIMEPOINT
from edc_form_validators import FormValidatorMixin
from edc_sites.forms import SiteModelFormMixin
from edc_visit_tracking.constants import MISSED_VISIT, SCHEDULED, UNSCHEDULED
from edc_visit_tracking.form_validators import VisitFormValidator

from ..models import SubjectVisit


class SubjectVisitFormValidator(VisitFormValidator):
    validate_missed_visit_reason = False

    def clean(self):
        super().clean()
        self.m2m_other_specify(
            OTHER,
            m2m_field="clinic_services",
            field_other="clinic_services_other",
        )

        self.validate__clinic_services()

        self.applicable_if(
            SCHEDULED, UNSCHEDULED, field="reason", field_applicable="info_source"
        )

    def validate__clinic_services(self):
        selections = self.get_m2m_selected("clinic_services")
        if (
            self.cleaned_data.get("appointment").visit_code_sequence == 0
            and self.cleaned_data.get("reason") != MISSED_VISIT
            and STUDY_DEFINED_TIMEPOINT not in selections
        ):
            raise forms.ValidationError({"clinic_services": "This is scheduled study visit."})
        elif (
            self.cleaned_data.get("appointment").visit_code_sequence != 0
            and self.cleaned_data.get("reason") != MISSED_VISIT
            and STUDY_DEFINED_TIMEPOINT in selections
        ):
            raise forms.ValidationError(
                {"clinic_services": "This is not a scheduled study visit."}
            )

        if self.cleaned_data.get("reason") == MISSED_VISIT:
            self.m2m_single_selection_if(NOT_APPLICABLE, m2m_field="clinic_services")

            self.m2m_selection_expected(
                m2m_field="clinic_services",
                response=NOT_APPLICABLE,
                error_msg=(
                    "Expected 'Not applicable' response (if this is a missed visit report)."
                ),
            )

        self.m2m_applicable_if_true(
            self.cleaned_data.get("reason") != MISSED_VISIT,
            m2m_field="clinic_services",
        )


class SubjectVisitForm(SiteModelFormMixin, FormValidatorMixin, forms.ModelForm):

    form_validator_cls = SubjectVisitFormValidator

    class Meta:
        model = SubjectVisit
        fields = "__all__"



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple
from edc_form_label.form_label_modeladmin_mixin import FormLabelModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin

from ..admin_site import mocca_subject_admin
from ..forms import HivReviewForm
from ..models import HivReview
from .fieldsets import care_delivery_fieldset_tuple
from .modeladmin_mixins import CrfModelAdminMixin


@admin.register(HivReview, site=mocca_subject_admin)
class HivReviewAdmin(CrfModelAdminMixin, FormLabelModelAdminMixin, SimpleHistoryAdmin):

    form = HivReviewForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        care_delivery_fieldset_tuple,
        (
            "Anit-retroviral therapy (ART)",
            {"fields": ("arv_initiated", "arv_initiation_actual_date")},
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "arv_initiated": admin.VERTICAL,
        "care_delivery": admin.VERTICAL,
        "crf_status": admin.VERTICAL,
        "dx": admin.VERTICAL,
    }



from django import forms
from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist
from edc_constants.constants import FEMALE, YES
from edc_form_validators import FormValidator, FormValidatorMixin
from edc_screening.modelform_mixins import AlreadyConsentedFormMixin

from ..models import MoccaRegister, SubjectScreening
from .care_status_form import CareStatusFormValidatorMixin


class SubjectScreeningFormValidator(CareStatusFormValidatorMixin, FormValidator):
    def clean(self) -> None:
        self.validate_is_mocca_participant()
        self.validate_consents_to_screen()
        self.validate_care_options()
        self.applicable_if(
            YES, field="screening_consent", field_applicable="willing_to_consent"
        )
        self.validate_pregnancy()
        self.applicable_if(
            YES,
            field="willing_to_consent",
            field_applicable="requires_acute_care",
            inverse=False,
        )
        self.validate_mocca_enrollment_data()
        return None

    def validate_is_mocca_participant(self) -> None:
        if self.cleaned_data.get("mocca_participant") != YES:
            raise forms.ValidationError(
                {
                    "mocca_participant": (
                        "Subject must have been a participant in the original MOCCA trial."
                    )
                }
            )
        return None

    def validate_consents_to_screen(self) -> None:
        if (
            not self.cleaned_data.get("screening_consent")
            or self.cleaned_data.get("screening_consent") != YES
        ):
            raise forms.ValidationError(
                {
                    "screening_consent": (
                        "You may NOT screen this subject without their verbal consent."
                    )
                }
            )
        return None

    def validate_pregnancy(self) -> None:
        if self.cleaned_data.get("mocca_register"):
            gender = self.cleaned_data.get("mocca_register").gender
        else:
            gender = self.instance.mocca_register.gender
        self.applicable_if_true(
            self.cleaned_data.get("willing_to_consent") == YES and gender == FEMALE,
            field_applicable="pregnant",
            inverse=False,
        )
        return None

    def validate_mocca_study_identifier_with_site(self) -> MoccaRegister:
        mocca_register = None
        mocca_register_cls = django_apps.get_model("mocca_screening.moccaregister")
        if self.cleaned_data.get("mocca_study_identifier") and self.cleaned_data.get(
            "mocca_site"
        ):
            mocca_study_identifier = self.cleaned_data.get("mocca_study_identifier")
            try:
                mocca_register = mocca_register_cls.objects.get(
                    mocca_study_identifier=mocca_study_identifier,
                )
            except ObjectDoesNotExist:
                raise forms.ValidationError(
                    {
                        "mocca_study_identifier": (
                            "Invalid MOCCA (original) study identifier. "
                            f"Got {mocca_study_identifier}"
                        )
                    }
                )
            else:
                if mocca_register.mocca_site != self.cleaned_data.get("mocca_site"):
                    expected_value = mocca_register.mocca_site
                    value = self.cleaned_data.get("mocca_site")
                    if expected_value != value:
                        raise forms.ValidationError(
                            {
                                "mocca_site": (
                                    f"Invalid MOCCA (original) site for given study "
                                    f"identifier. Got `{expected_value}` != `{value}` for "
                                    f"`{mocca_study_identifier}`"
                                )
                            }
                        )
        return mocca_register

    def validate_mocca_enrollment_data(self) -> None:
        """Raises an exception if either the birth year, gender or initials
        do not match the register record for the given
        `mocca_study_identifier`.
        """
        mocca_register = self.validate_mocca_study_identifier_with_site()
        if (
            mocca_register
            and self.cleaned_data.get("birth_year")
            and self.cleaned_data.get("initials")
        ):
            for attrname in ["initials", "gender", "birth_year"]:
                expected_value = getattr(mocca_register, attrname)
                value = self.cleaned_data.get(attrname)
                if expected_value != value:
                    label = attrname.replace("_", " ")
                    raise forms.ValidationError(
                        {
                            attrname: (
                                f"Invalid `{label}` for this MOCCA (original) study "
                                f"identifier. Got `{expected_value}` != `{value}` for "
                                f"`{mocca_register.mocca_study_identifier}`"
                            )
                        }
                    )

        if (
            self.cleaned_data.get("age_in_years")
            and self.cleaned_data.get("birth_year")
            and self.cleaned_data.get("report_datetime")
        ):
            expected_age = self.cleaned_data.get(
                "report_datetime"
            ).year - self.cleaned_data.get("birth_year")
            if abs(expected_age - self.cleaned_data.get("age_in_years")) > 1:
                raise forms.ValidationError(
                    {"age_in_years": "Does not make sense relative to birth year given"}
                )
        return None


class SubjectScreeningForm(AlreadyConsentedFormMixin, FormValidatorMixin, forms.ModelForm):
    form_validator_cls = SubjectScreeningFormValidator

    class Meta:
        model = SubjectScreening
        fields = "__all__"



# Generated by Django 3.0.9 on 2020-12-17 03:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0009_auto_20201216_0454"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalmoccaregister",
            name="screen_now",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                help_text="Only select 'yes' if the patient is present in the clinic now.",
                max_length=15,
                verbose_name="Patient is present. Screen now instead of calling?",
            ),
        ),
        migrations.AlterField(
            model_name="moccaregister",
            name="screen_now",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="No",
                help_text="Only select 'yes' if the patient is present in the clinic now.",
                max_length=15,
                verbose_name="Patient is present. Screen now instead of calling?",
            ),
        ),
    ]



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from edc_model import models as edc_models
from edc_reportable import CELLS_PER_MILLIMETER_CUBED_DISPLAY

from ..model_mixins import CrfModelMixin


class Cd4Result(CrfModelMixin, edc_models.BaseUuidModel):

    drawn_date = models.DateField(
        verbose_name="Specimen collection date",
        validators=[edc_models.date_not_future],
    )

    result = models.IntegerField(
        verbose_name="CD4 Result",
        validators=[MinValueValidator(0), MaxValueValidator(3000)],
        help_text=f"in {CELLS_PER_MILLIMETER_CUBED_DISPLAY}",
    )

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "CD4 Result"
        verbose_name_plural = "CD4 Results"



from django.test import TestCase, tag
from edc_utils import get_utcnow
from edc_visit_schedule.constants import DAY1
from edc_visit_tracking.constants import SCHEDULED, UNSCHEDULED

from ..mocca_test_case_mixin import MoccaTestCaseMixin


class TestVisitSchedule(MoccaTestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.subject_screening = self.get_subject_screening(report_datetime=get_utcnow())
        self.subject_consent = self.get_subject_consent(
            subject_screening=self.subject_screening
        )

    def test_baseline(self):
        subject_visit = self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
            visit_code=DAY1,
            reason=SCHEDULED,
        )
        self.assertEqual("1000", subject_visit.appointment.visit_code)
        self.assertEqual(0, subject_visit.appointment.visit_code_sequence)

    def test_next_up_to_12m(self):
        subject_visit = self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
            visit_code=DAY1,
            reason=SCHEDULED,
        )

        for i in range(10, 130, 10):
            subject_visit = self.get_next_subject_visit(
                subject_visit=subject_visit,
                reason=SCHEDULED,
            )
            visit_code = str(1000 + i)
            self.assertEqual(visit_code, subject_visit.appointment.visit_code)
            self.assertEqual(0, subject_visit.appointment.visit_code_sequence)

    def test_next_unscheduled(self):
        subject_visit = self.get_subject_visit(
            subject_screening=self.subject_screening,
            subject_consent=self.subject_consent,
            visit_code=DAY1,
            reason=SCHEDULED,
        )

        subject_visit = self.get_next_subject_visit(
            subject_visit=subject_visit,
            reason=UNSCHEDULED,
        )
        self.assertEqual("1000", subject_visit.appointment.visit_code)
        self.assertEqual(1, subject_visit.appointment.visit_code_sequence)

        subject_visit = self.get_next_subject_visit(
            subject_visit=subject_visit,
            reason=UNSCHEDULED,
        )

        self.assertEqual("1000", subject_visit.appointment.visit_code)
        self.assertEqual(2, subject_visit.appointment.visit_code_sequence)

        subject_visit = self.get_next_subject_visit(
            subject_visit=subject_visit,
            reason=SCHEDULED,
        )

        self.assertEqual("1010", subject_visit.appointment.visit_code)
        self.assertEqual(0, subject_visit.appointment.visit_code_sequence)



# Generated by Django 3.1.6 on 2021-02-28 21:10

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mocca_subject', '0010_auto_20210228_2319'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='dminitialreview',
            name='glucose_performed',
        ),
        migrations.RemoveField(
            model_name='historicaldminitialreview',
            name='glucose_performed',
        ),
    ]



# Generated by Django 3.0.9 on 2020-12-03 00:38

import uuid

import _socket
import django.core.validators
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_crypto_fields.fields.encrypted_char_field
import django_crypto_fields.fields.firstname_field
import django_crypto_fields.fields.lastname_field
import django_revision.revision_field
import edc_sites.models
import edc_utils.date
import simple_history.models
from django.conf import settings
from django.db import migrations, models

import mocca_screening.mocca_original_sites
import mocca_screening.models.mocca_register


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("mocca_lists", "0001_initial"),
        ("sites", "0002_alter_domain_unique"),
        ("mocca_screening", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="moccaregister",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", mocca_screening.models.mocca_register.Manager()),
            ],
        ),
        migrations.AddField(
            model_name="moccaregister",
            name="site",
            field=models.ForeignKey(
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.Site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="mocca_study_identifier",
            field=models.CharField(
                db_index=True,
                help_text="Format must match original identifier. e.g. 12-3456 for UG, 123456 for TZ",
                max_length=25,
                validators=[
                    django.core.validators.RegexValidator(
                        "0[0-9]{1}\\-0[0-9]{3}|[0-9]{6}",
                        "Invalid format. Expected 12-3456 for UG, 123456 for TZ",
                    )
                ],
                verbose_name="Original MOCCA study identifier",
            ),
        ),
        migrations.AlterField(
            model_name="moccaregister",
            name="initials",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text="Use UPPERCASE letters only. May be 2 or 3 letters. (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[
                    django.core.validators.RegexValidator("[A-Z]{1,3}", "Invalid format"),
                    django.core.validators.MinLengthValidator(2),
                    django.core.validators.MaxLengthValidator(3),
                ],
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="mocca_study_identifier",
            field=models.CharField(
                help_text="Format must match original identifier. e.g. 12-3456 for UG, 123456 for TZ",
                max_length=25,
                unique=True,
                validators=[
                    django.core.validators.RegexValidator(
                        "0[0-9]{1}\\-0[0-9]{3}|[0-9]{6}",
                        "Invalid format. Expected 12-3456 for UG, 123456 for TZ",
                    )
                ],
                verbose_name="Original MOCCA study identifier",
            ),
        ),
        migrations.CreateModel(
            name="HistoricalMoccaRegister",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "report_datetime",
                    models.DateTimeField(default=edc_utils.date.get_utcnow),
                ),
                (
                    "mocca_screening_identifier",
                    models.CharField(
                        blank=True,
                        help_text="If known",
                        max_length=15,
                        null=True,
                        verbose_name="MOCCA (original) screening identifier",
                    ),
                ),
                (
                    "mocca_study_identifier",
                    models.CharField(
                        help_text="Format must match original identifier. e.g. 12-3456 for UG, 123456 for TZ",
                        max_length=25,
                        validators=[
                            django.core.validators.RegexValidator(
                                "0[0-9]{1}\\-0[0-9]{3}|[0-9]{6}",
                                "Invalid format. Expected 12-3456 for UG, 123456 for TZ",
                            )
                        ],
                        verbose_name="MOCCA (original) study identifier",
                    ),
                ),
                (
                    "mocca_country",
                    models.CharField(
                        choices=[("uganda", "Uganda"), ("tanzania", "Tanzania")],
                        max_length=25,
                    ),
                ),
                (
                    "first_name",
                    django_crypto_fields.fields.firstname_field.FirstnameField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                    ),
                ),
                (
                    "last_name",
                    django_crypto_fields.fields.lastname_field.LastnameField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                    ),
                ),
                (
                    "initials",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text="Use UPPERCASE letters only. May be 2 or 3 letters. (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[
                            django.core.validators.RegexValidator(
                                "[A-Z]{1,3}", "Invalid format"
                            ),
                            django.core.validators.MinLengthValidator(2),
                            django.core.validators.MaxLengthValidator(3),
                        ],
                    ),
                ),
                (
                    "gender",
                    models.CharField(choices=[("M", "Male"), ("F", "Female")], max_length=10),
                ),
                (
                    "age_in_years",
                    models.IntegerField(
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(18),
                            django.core.validators.MaxValueValidator(110),
                        ],
                    ),
                ),
                (
                    "birth_year",
                    models.IntegerField(
                        null=True,
                        validators=[
                            django.core.validators.MinValueValidator(1900),
                            django.core.validators.MaxValueValidator(2002),
                        ],
                    ),
                ),
                ("dob", models.DateField(blank=True, null=True)),
                (
                    "survival_status",
                    models.CharField(
                        choices=[
                            ("alive", "Alive"),
                            ("dead", "Deceased"),
                            ("unknown", "Unknown"),
                        ],
                        default="unknown",
                        max_length=25,
                    ),
                ),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "mocca_site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        limit_choices_to=mocca_screening.mocca_original_sites.get_mocca_site_limited_to,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="mocca_lists.MoccaOriginalSites",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.Site",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical MOCCA Patient Register",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
    ]



from django import forms
from edc_constants.constants import OTHER
from edc_form_validators import FormValidator, FormValidatorMixin

from ..models import SubjectRefusal


class SubjectRefusalScreeningFormValidator(FormValidator):
    def clean(self):
        self.required_if(OTHER, field="reason", field_required="other_reason")


class SubjectRefusalScreeningForm(FormValidatorMixin, forms.ModelForm):
    form_validator_cls = SubjectRefusalScreeningFormValidator

    class Meta:
        model = SubjectRefusal
        fields = "__all__"



from copy import copy

from django.contrib import admin
from edc_action_item import action_fields, action_fieldset_tuple
from edc_model_admin import SimpleHistoryAdmin, audit_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from ..admin_site import mocca_prn_admin
from ..forms import ProtocolDeviationViolationForm
from ..models import ProtocolDeviationViolation


@admin.register(ProtocolDeviationViolation, site=mocca_prn_admin)
class ProtocolDeviationViolationAdmin(ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin):

    form = ProtocolDeviationViolationForm

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_identifier",
                    "report_datetime",
                    "short_description",
                    "report_type",
                )
            },
        ),
        (
            "Details of protocol violation",
            {
                "description": (
                    "The following questions are only required if "
                    "this is a protocol violation."
                ),
                "fields": (
                    "safety_impact",
                    "safety_impact_details",
                    "study_outcomes_impact",
                    "study_outcomes_impact_details",
                    "violation_datetime",
                    "violation_type",
                    "violation_type_other",
                    "violation_description",
                    "violation_reason",
                ),
            },
        ),
        (
            "Actions taken",
            {
                "fields": (
                    "corrective_action_datetime",
                    "corrective_action",
                    "preventative_action_datetime",
                    "preventative_action",
                    "action_required",
                )
            },
        ),
        ("Report status", {"fields": ("report_status", "report_closed_datetime")}),
        action_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "action_required": admin.VERTICAL,
        "report_status": admin.VERTICAL,
        "report_type": admin.VERTICAL,
        "safety_impact": admin.VERTICAL,
        "study_outcomes_impact": admin.VERTICAL,
        "violation_type": admin.VERTICAL,
    }

    list_display = (
        "subject_identifier",
        "dashboard",
        "short_description",
        "report_datetime",
        "status",
        "action_required",
        "report_type",
        "tracking_identifier",
        "action_identifier",
        "user_created",
    )

    list_filter = ("action_required", "report_status", "report_type")

    search_fields = [
        "subject_identifier",
        "action_identifier",
        "short_description",
        "tracking_identifier",
    ]

    def get_readonly_fields(self, request, obj=None):
        fields = super().get_readonly_fields(request, obj)
        action_flds = copy(list(action_fields))
        action_flds.remove("action_identifier")
        fields = list(action_flds) + list(fields)
        return fields

    def status(self, obj=None):
        return obj.report_status.title()



from django.db import models
from edc_model.models import BaseUuidModel, OtherCharField
from edc_model.models.historical_records import HistoricalRecords
from edc_search.model_mixins import SearchSlugManager
from edc_sites.models import CurrentSiteManager, SiteModelMixin
from edc_utils import get_utcnow

from ..choices import REFUSAL_REASONS_SCREENING


class Manager(SearchSlugManager, models.Manager):
    use_in_migrations = True

    def get_by_natural_key(self, mocca_register):
        return self.get(mocca_register=mocca_register)


class SubjectRefusalScreening(SiteModelMixin, BaseUuidModel):
    mocca_register = models.OneToOneField(
        "mocca_screening.moccaregister",
        on_delete=models.PROTECT,
        null=True,
        verbose_name="MOCCA (original) register details",
    )

    report_datetime = models.DateTimeField(
        verbose_name="Report Date and Time", default=get_utcnow
    )

    reason = models.CharField(
        verbose_name="Reason for refusal to screen",
        max_length=25,
        choices=REFUSAL_REASONS_SCREENING,
    )

    other_reason = OtherCharField()

    comment = models.TextField(
        verbose_name="Additional Comments",
        null=True,
        blank=True,
    )

    on_site = CurrentSiteManager()

    objects = Manager()

    history = HistoricalRecords()

    def __str__(self):
        return self.mocca_register.mocca_study_identifier

    def natural_key(self):
        return tuple(self.mocca_register)

    @staticmethod
    def get_search_slug_fields():
        return ["screening_identifier"]

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Refusal to Screen"
        verbose_name_plural = "Refusal to Screen"



from edc_offstudy.constants import END_OF_STUDY_ACTION
from edc_visit_schedule import site_visit_schedules

from mocca_visit_schedule.constants import SCHEDULE

from .end_of_study import EndOfStudy


class OffSchedule(EndOfStudy):
    action_name = END_OF_STUDY_ACTION

    tracking_identifier_prefix = "OH"

    def save(self, *args, **kwargs):
        self.visit_schedule_name = "visit_schedule"
        self.schedule_name = SCHEDULE
        super().save(*args, **kwargs)

    @property
    def visit_schedule(self):
        """Returns a visit schedule object."""
        return site_visit_schedules.get_by_onschedule_model(
            onschedule_model=self._meta.label_lower
        )[0]

    @property
    def schedule(self):
        """Returns a schedule object."""
        return site_visit_schedules.get_by_onschedule_model(
            onschedule_model=self._meta.label_lower
        )[1]

    class Meta:
        proxy = True
        verbose_name = "End of Study"
        verbose_name_plural = "End of Study"



from django import forms
from edc_constants.constants import NO, YES
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_dx.form_validators import DiagnosisFormValidatorMixin
from edc_dx_review.utils import requires_clinical_review_at_baseline
from edc_form_validators.form_validator import FormValidator

from ..models import ClinicalReview


class ClinicalReviewFormValidator(
    DiagnosisFormValidatorMixin, CrfFormValidatorMixin, FormValidator
):
    def clean(self):
        requires_clinical_review_at_baseline(self.cleaned_data.get("subject_visit"))

        diagnoses = self.get_diagnoses()

        for cond, label in [
            ("htn", "hypertension"),
            ("dm", "diabetes"),
            ("hiv", "HIV"),
            ("chol", "High Cholesterol"),
        ]:
            self.applicable_if_not_diagnosed(
                diagnoses=diagnoses,
                prefix=cond,
                field_applicable=f"{cond}_test",
                label=label,
            )
            self.required_if(YES, field=f"{cond}_test", field_required=f"{cond}_test_date")
            self.required_if(YES, field=f"{cond}_test", field_required=f"{cond}_reason")
            self.applicable_if(YES, field=f"{cond}_test", field_applicable=f"{cond}_dx")

        self.required_if(
            YES,
            field="health_insurance",
            field_required="health_insurance_monthly_pay",
            field_required_evaluate_as_int=True,
        )
        self.required_if(
            YES,
            field="patient_club",
            field_required="patient_club_monthly_pay",
            field_required_evaluate_as_int=True,
        )

    def raise_if_dx_and_applicable(self, clinic, cond):
        if self.subject_screening.clinic_type in [clinic] and self.cleaned_data.get(
            f"{cond}_test"
        ) in [YES, NO]:
            raise forms.ValidationError(
                {
                    f"{cond}_test": (
                        f"Not applicable. Patient was recruited from the {cond.title} clinic."
                    ),
                }
            )


class ClinicalReviewForm(CrfModelFormMixin, forms.ModelForm):

    form_validator_cls = ClinicalReviewFormValidator

    class Meta:
        model = ClinicalReview
        fields = "__all__"



from .care_status_admin import CareStatusAdmin
from .daily_closing_log_admin import DailyClosingLogAdmin
from .mocca_register_admin import MoccaRegisterAdmin
from .mocca_register_contact_admin import MoccaRegisterContactAdmin
from .subject_refusal_admin import SubjectRefusalAdmin
from .subject_refusal_screening_admin import SubjectRefusalScreeningAdmin
from .subject_screening_admin import SubjectScreeningAdmin



from django.db import models
from edc_consent.model_mixins import RequiresConsentFieldsModelMixin
from edc_constants.constants import NOT_APPLICABLE
from edc_metadata.model_mixins.creates import CreatesMetadataModelMixin
from edc_model import models as edc_models
from edc_reference.model_mixins import ReferenceModelMixin
from edc_sites.models import CurrentSiteManager as BaseCurrentSiteManager
from edc_sites.models import SiteModelMixin
from edc_visit_tracking.managers import VisitModelManager
from edc_visit_tracking.model_mixins import VisitModelMixin

from mocca_lists.models import ClinicServices

from ..choices import INFO_SOURCE, VISIT_REASON, VISIT_UNSCHEDULED_REASON
from .subject_visit_missed import SubjectVisitMissed


class CurrentSiteManager(VisitModelManager, BaseCurrentSiteManager):
    pass


class SubjectVisit(
    VisitModelMixin,
    ReferenceModelMixin,
    CreatesMetadataModelMixin,
    SiteModelMixin,
    RequiresConsentFieldsModelMixin,
    edc_models.BaseUuidModel,
):
    """A model completed by the user that captures the covering
    information for the data collected for this timepoint/appointment,
    e.g.report_datetime.
    """

    reason = models.CharField(
        verbose_name="What is the reason for this visit report?",
        max_length=25,
        choices=VISIT_REASON,
        help_text=(
            "Only baseline (0m), 6m and 12m are considered "
            "`scheduled` visits as per the MOCCA protocol."
            f"If `missed' complete CRF {SubjectVisitMissed._meta.verbose_name}."
        ),
    )

    reason_unscheduled = models.CharField(
        verbose_name="If 'unscheduled', provide reason for the unscheduled visit",
        max_length=25,
        choices=VISIT_UNSCHEDULED_REASON,
        default=NOT_APPLICABLE,
    )

    clinic_services = models.ManyToManyField(
        ClinicServices,
        verbose_name="Why is the patient at the clinic today?",
        related_name="visit_clinic_services",
    )

    clinic_services_other = edc_models.OtherCharField()

    info_source = models.CharField(
        verbose_name="What is the main source of this information?",
        max_length=25,
        choices=INFO_SOURCE,
    )

    on_site = CurrentSiteManager()

    objects = VisitModelManager()

    history = edc_models.HistoricalRecords()

    class Meta(VisitModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        pass



from django.conf import settings
from django.db import models
from edc_dx_review.model_mixins import (
    ClinicalReviewBaselineCholModelMixin,
    ClinicalReviewBaselineDmModelMixin,
    ClinicalReviewBaselineHivModelMixin,
    ClinicalReviewBaselineHtnModelMixin,
    ClinicalReviewBaselineModelMixin,
)
from edc_model import models as edc_models

from ..model_mixins import CrfModelMixin


class ClinicalReviewBaseline(
    ClinicalReviewBaselineHivModelMixin,
    ClinicalReviewBaselineHtnModelMixin,
    ClinicalReviewBaselineDmModelMixin,
    ClinicalReviewBaselineCholModelMixin,
    ClinicalReviewBaselineModelMixin,
    CrfModelMixin,
    edc_models.BaseUuidModel,
):
    subject_visit = models.OneToOneField(
        settings.SUBJECT_VISIT_MODEL,
        on_delete=models.PROTECT,
        related_name="+",
    )

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Clinical Review: Baseline"
        verbose_name_plural = "Clinical Review: Baseline"



from datetime import datetime
from typing import Protocol, Type

from edc_model.stubs import BaseUuidModelStub
from edc_screening.stubs import (
    SubjectScreeningModelStub as BaseSubjectScreeningModelStub,
)


class MoccaSiteStub(BaseUuidModelStub, Protocol):
    name: str


class MoccaRegisterModelStub(BaseUuidModelStub, Protocol):
    age_in_years: int
    birth_year: int
    gender: str
    mocca_site: MoccaSiteStub
    mocca_study_identfier: str
    report_datetime: datetime
    screening_identifier: str


class MoccaRegisterFormStub(Protocol):
    cleaned_data: dict
    instance: MoccaRegisterModelStub
    model: Type[MoccaRegisterModelStub]

    def clean(self) -> dict:
        ...


class MoccaRegisterFormValidatorStub(MoccaRegisterFormStub, Protocol):
    def get_age(self) -> int:
        ...


class SubjectScreeningModelStub(BaseSubjectScreeningModelStub, Protocol):
    care: str
    mocca_register: MoccaRegisterModelStub
    pregnant: str
    requires_acute_care: str
    unsuitable_for_study: str
    willing_to_consent: str



from django import forms
from edc_action_item.forms import ActionItemFormMixin
from edc_form_validators import FormValidatorMixin
from edc_sites.forms import SiteModelFormMixin
from edc_visit_schedule.modelform_mixins import OffScheduleModelFormMixin

from ..form_validators import EndOfStudyFormValidator
from ..models import EndOfStudy


class EndOfStudyForm(
    SiteModelFormMixin,
    FormValidatorMixin,
    ActionItemFormMixin,
    OffScheduleModelFormMixin,
    forms.ModelForm,
):

    form_validator_cls = EndOfStudyFormValidator

    subject_identifier = forms.CharField(
        label="Subject Identifier",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    visit_schedule = forms.CharField(
        label="Schedule",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )
    schedule = forms.CharField(
        label="Schedule",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    class Meta:
        model = EndOfStudy
        fields = "__all__"
        labels = {"offschedule_datetime": "Date patient terminated on study:"}



from edc_model import models as edc_models

from ..model_mixins import CrfModelMixin
from .model_mixins import (
    CholMedicationsModelMixin,
    DmMedicationsModelMixin,
    HivMedicationsModelMixin,
    HtnMedicationsModelMixin,
)


class Medications(
    HivMedicationsModelMixin,
    HtnMedicationsModelMixin,
    DmMedicationsModelMixin,
    CholMedicationsModelMixin,
    CrfModelMixin,
    edc_models.BaseUuidModel,
):
    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Medications"
        verbose_name_plural = "Medications"



from django import forms
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_phq9.forms import Phq9FormValidator

from ..models import PatientHealth


class PatientHealthForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = Phq9FormValidator

    class Meta:
        model = PatientHealth
        fields = "__all__"



# Generated by Django 3.1.7 on 2021-03-26 17:16

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mocca_subject', '0018_auto_20210302_0757'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='historicalbloodresultsfbc',
            name='action_item',
        ),
        migrations.RemoveField(
            model_name='historicalbloodresultsfbc',
            name='fbc_requisition',
        ),
        migrations.RemoveField(
            model_name='historicalbloodresultsfbc',
            name='history_user',
        ),
        migrations.RemoveField(
            model_name='historicalbloodresultsfbc',
            name='parent_action_item',
        ),
        migrations.RemoveField(
            model_name='historicalbloodresultsfbc',
            name='related_action_item',
        ),
        migrations.RemoveField(
            model_name='historicalbloodresultsfbc',
            name='site',
        ),
        migrations.RemoveField(
            model_name='historicalbloodresultsfbc',
            name='subject_visit',
        ),
        migrations.AddField(
            model_name='bloodresultslipid',
            name='chol',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=8, null=True, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(999)], verbose_name='Cholesterol'),
        ),
        migrations.AddField(
            model_name='bloodresultslipid',
            name='chol_abnormal',
            field=models.CharField(blank=True, choices=[('Yes', 'Yes'), ('No', 'No')], max_length=25, null=True, verbose_name='abnormal'),
        ),
        migrations.AddField(
            model_name='bloodresultslipid',
            name='chol_reportable',
            field=models.CharField(blank=True, choices=[('N/A', 'Not applicable'), ('3', 'Yes, grade 3'), ('4', 'Yes, grade 4'), ('No', 'Not reportable'), ('Already reported', 'Already reported'), ('present_at_baseline', 'Present at baseline')], max_length=25, null=True, verbose_name='reportable'),
        ),
        migrations.AddField(
            model_name='bloodresultslipid',
            name='chol_units',
            field=models.CharField(blank=True, choices=[('mmol/L', 'mmol/L')], max_length=15, null=True, verbose_name='units'),
        ),
        migrations.AddField(
            model_name='historicalbloodresultslipid',
            name='chol',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=8, null=True, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(999)], verbose_name='Cholesterol'),
        ),
        migrations.AddField(
            model_name='historicalbloodresultslipid',
            name='chol_abnormal',
            field=models.CharField(blank=True, choices=[('Yes', 'Yes'), ('No', 'No')], max_length=25, null=True, verbose_name='abnormal'),
        ),
        migrations.AddField(
            model_name='historicalbloodresultslipid',
            name='chol_reportable',
            field=models.CharField(blank=True, choices=[('N/A', 'Not applicable'), ('3', 'Yes, grade 3'), ('4', 'Yes, grade 4'), ('No', 'Not reportable'), ('Already reported', 'Already reported'), ('present_at_baseline', 'Present at baseline')], max_length=25, null=True, verbose_name='reportable'),
        ),
        migrations.AddField(
            model_name='historicalbloodresultslipid',
            name='chol_units',
            field=models.CharField(blank=True, choices=[('mmol/L', 'mmol/L')], max_length=15, null=True, verbose_name='units'),
        ),
        migrations.DeleteModel(
            name='BloodResultsFbc',
        ),
        migrations.DeleteModel(
            name='HistoricalBloodResultsFbc',
        ),
    ]



from django import forms
from edc_constants.constants import YES
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_form_validators.form_validator import FormValidator
from edc_model.utils import estimated_date_from_ago
from edc_visit_schedule.utils import raise_if_not_baseline

from ..models import ClinicalReviewBaseline


class ClinicalReviewBaselineFormValidator(CrfFormValidatorMixin, FormValidator):
    def clean(self):
        raise_if_not_baseline(self.cleaned_data.get("subject_visit"))
        estimated_date_from_ago(data=self.cleaned_data, ago_field="hiv_test_ago")
        self.when_tested_required(cond="hiv")

        for cond in ["htn", "dm", "chol"]:
            estimated_date_from_ago(data=self.cleaned_data, ago_field=f"{cond}_test_ago")
            self.when_tested_required(cond=cond)
            self.required_if(YES, field=f"{cond}_test", field_required=f"{cond}_dx")

    def when_tested_required(self, cond=None):
        if self.cleaned_data.get(f"{cond}_test") == YES:
            if not self.cleaned_data.get(f"{cond}_test_ago") and not self.cleaned_data.get(
                f"{cond}_test_date"
            ):
                raise forms.ValidationError(
                    f"{cond.title()}: When was the subject tested? Either provide an "
                    "estimated time 'ago' or provide the exact date. See below."
                )


class ClinicalReviewBaselineForm(CrfModelFormMixin, forms.ModelForm):

    form_validator_cls = ClinicalReviewBaselineFormValidator

    class Meta:
        model = ClinicalReviewBaseline
        fields = "__all__"



from edc_action_item.utils import register_actions
from edc_ltfu.action_items import LtfuAction
from edc_offstudy.action_items import EndOfStudyAction as BaseEndOfStudyAction
from edc_protocol_violation.action_items import (
    ProtocolDeviationViolationAction as BaseProtocolDeviationViolationAction,
)


class EndOfStudyAction(BaseEndOfStudyAction):

    reference_model = "mocca_prn.endofstudy"
    admin_site_name = "mocca_prn_admin"


class LossToFollowupAction(LtfuAction):

    admin_site_name = "mocca_prn_admin"


class ProtocolDeviationViolationAction(BaseProtocolDeviationViolationAction):
    reference_model = "mocca_prn.protocoldeviationviolation"
    admin_site_name = "mocca_prn_admin"


register_actions(
    EndOfStudyAction,
    ProtocolDeviationViolationAction,
    LossToFollowupAction,
)



from edc_action_item.auth_objects import (
    ACTION_ITEM,
    ACTION_ITEM_EXPORT,
    ACTION_ITEM_VIEW_ONLY,
)
from edc_adverse_event.auth_objects import AE, AE_REVIEW, AE_SUPER
from edc_appointment.auth_objects import (
    APPOINTMENT,
    APPOINTMENT_EXPORT,
    APPOINTMENT_VIEW,
)
from edc_auth.auth_objects import (
    AUDITOR,
    AUDITOR_ROLE,
    CLINIC,
    CLINIC_SUPER,
    CLINICIAN_ROLE,
    CLINICIAN_SUPER_ROLE,
    NURSE_ROLE,
)
from edc_auth.site_auths import site_auths
from edc_data_manager.auth_objects import (
    DATA_MANAGER_EXPORT,
    DATA_MANAGER_ROLE,
    SITE_DATA_MANAGER_ROLE,
)
from edc_export.auth_objects import DATA_EXPORTER_ROLE
from edc_offstudy.auth_objects import OFFSTUDY
from edc_screening.auth_objects import SCREENING, SCREENING_VIEW

from .auth_objects import clinic_codenames

site_auths.update_group(*clinic_codenames, name=CLINIC, no_delete=True)
site_auths.update_group(*clinic_codenames, name=CLINIC_SUPER)
site_auths.update_group(*clinic_codenames, name=AUDITOR, view_only=True)

# update edc roles
site_auths.update_role(
    ACTION_ITEM,
    AE,
    APPOINTMENT,
    CLINIC,
    name=CLINICIAN_ROLE,
)

site_auths.update_role(
    ACTION_ITEM,
    AE_SUPER,
    APPOINTMENT,
    CLINIC_SUPER,
    name=CLINICIAN_SUPER_ROLE,
)

site_auths.update_role(
    ACTION_ITEM,
    AE,
    APPOINTMENT,
    CLINIC,
    name=NURSE_ROLE,
)

site_auths.update_role(
    ACTION_ITEM,
    AE,
    APPOINTMENT,
    CLINIC,
    CLINIC,
    OFFSTUDY,
    SCREENING,
    name=DATA_MANAGER_ROLE,
)

site_auths.update_role(
    ACTION_ITEM_VIEW_ONLY,
    AE_REVIEW,
    APPOINTMENT_VIEW,
    AUDITOR,
    name=AUDITOR_ROLE,
)

site_auths.update_role(
    AUDITOR,
    ACTION_ITEM,
    AE_REVIEW,
    AUDITOR,
    SCREENING_VIEW,
    name=SITE_DATA_MANAGER_ROLE,
)

# data export
site_auths.update_role(
    ACTION_ITEM_EXPORT,
    APPOINTMENT_EXPORT,
    DATA_MANAGER_EXPORT,
    name=DATA_EXPORTER_ROLE,
)



from django.contrib import admin
from edc_list_data.admin import ListModelAdminMixin

from ..admin_site import mocca_subject_admin
from ..models import Rx


@admin.register(Rx, site=mocca_subject_admin)
class RxAdmin(ListModelAdminMixin, admin.ModelAdmin):
    """Declared since autocomplete needs a local modeladmin class.

    Ensure js is up-to-date (run collecstatic and purge CDN cache)"""

    pass



from django import forms
from edc_constants.constants import NO
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_dx_review.utils import raise_if_clinical_review_does_not_exist
from edc_form_validators.form_validator import FormValidator
from edc_glucose.form_validators import GlucoseFormValidatorMixin

from ..models import DmReview


class DmReviewFormValidator(
    GlucoseFormValidatorMixin,
    CrfFormValidatorMixin,
    FormValidator,
):
    def clean(self):
        raise_if_clinical_review_does_not_exist(self.cleaned_data.get("subject_visit"))
        self.validate_care_delivery()
        self.validate_glucose_test()

    def validate_care_delivery(self) -> None:
        self.required_if(NO, field="care_delivery", field_required="care_delivery_other")


class DmReviewForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = DmReviewFormValidator

    class Meta:
        model = DmReview
        fields = "__all__"
